/*
 * SPDX-FileCopyrightText: 2022 Pepijn de Vos
 *
 * SPDX-License-Identifier: MPL-2.0
 */

body, html {
    padding: 0;
    margin: 0;
    overflow: hidden;
}

h1 {
    font: 500 22px/28px 'Roboto', sans-serif;
    letter-spacing: 0.88px;
    color: #222222;
}

.bk-root {
    font: 400 16px/18px 'Roboto', sans-serif;
    color: #222222A5;
}
.bk-root .horizontal .bk-input-group {
    flex-direction: row;
    align-items: center;
    justify-content: space-between;
}
.bk-root .horizontal .bk-input-group label {
    flex: 0 0 200px;
}

.bk-root .bk-input-group label {
    text-align: left;
    font: 500 16px/18px 'Roboto', sans-serif;
    letter-spacing: 0.72px;
}

.bk-root .bk-input-group input,
.bk-root .bk-input-group select {
    background: #00000008 0% 0% no-repeat padding-box;
    border: 2px solid #2222220D;
    border-radius: 5px;
    padding: 10px 15px;
    opacity: 1;
    text-align: left;
    font: 400 16px/18px 'Roboto', sans-serif;
    letter-spacing: 0.72px;
    color: #222222;
}

.bk-root .bk-headers-wrapper {
    padding: 20px;
}
.bk-root .bk-tabs-header.bk-left .bk-headers {
    align-items: stretch;
}
.bk-root .bk-tabs-header.bk-left .bk-tab {
    border: none;
    text-align: left;
    font: 500 16px/18px 'Roboto', sans-serif;
    letter-spacing: 0.72px;
    padding: 10px;
}
.bk-root .bk-tabs-header.bk-left .bk-tab.bk-active {
    background: #0E1A32 0% 0% no-repeat padding-box;
    border-radius: 5px;
    opacity: 1;
    text-align: left;
    font: 500 16px/18px 'Roboto', sans-serif;
    letter-spacing: 0.72px;
    color: #FFFFFF;
    opacity: 1;
}

.bk-root .bk-btn {
    background: #2222220A 0% 0% no-repeat padding-box;
    box-shadow: 0px 1px 5px #00000026;
    border: none;
    border-radius: 5px;
    font: 500 16px/18px 'Roboto', sans-serif;
    letter-spacing: 0.9px;
    padding: 15px;
    text-transform: uppercase;
}
.bk-root .bk-btn:hover {
    background: #2222221A 0% 0% no-repeat padding-box;
}
.bk-root .bk-btn.bk-btn-primary {
    background: #0E1A32 0% 0% no-repeat padding-box;
    color: #FFFFFF;
}
.bk-root .bk-btn.bk-btn-primary:hover {
    color: #dddddd;
}

.bk-root .wizard-controls {
    outline: 1px solid #e6e6e6;
}
.bk-root .bk.spacer {
    display: none !important;
}