#!/bin/bash

# SPDX-FileCopyrightText: 2022 Pepijn de Vos
#
# SPDX-License-Identifier: MPL-2.0

curl -o common.js https://nyancad.github.io/Mosaic/app/js/common.js
curl -o editor.js https://nyancad.github.io/Mosaic/app/js/editor.js
curl -o libman.js https://nyancad.github.io/Mosaic/app/js/libman.js
curl -o style.css https://nyancad.github.io/Mosaic/app/css/style.css