<!--
SPDX-FileCopyrightText: 2022 Pepijn de Vos

SPDX-License-Identifier: MPL-2.0
-->

<!doctype html>
<html>

<head>
  <meta charset="utf-8" />
  <title>Mosaic</title>
  <link rel="preconnect" href="https://fonts.googleapis.com">
  <link rel="preconnect" href="https://fonts.gstatic.com" crossorigin>
  <link href="https://fonts.googleapis.com/css2?family=Roboto:wght@400;500;700&display=swap" rel="stylesheet"> 
  <link rel="stylesheet" href="{{ static_url('style.css') }}">
  <style>
    /* when not embedded, set html and body to 100% */
    html, body {
      width: 100%;
      height: 100%;
      overflow: hidden;
      margin: 0;
      padding: 0;
    }
  </style>
</head>

<body>
  <div id="mosaic_libman"></div>
  <script>
    {% if couchdb %}
    window.dburl = new URL("/couchdb/", window.location).href
    {% endif %}
  </script>
  <script src="{{ static_url('common.js') }}"></script>
  <script src="{{ static_url('libman.js') }}"></script>
</body>

</html>